/**
 * Copyright (C) cedarsoft GmbH.
 *
 * Licensed under the GNU General Public License version 3 (the "License")
 * with Classpath Exception; you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *         http://www.cedarsoft.org/gpl3ce
 *         (GPL 3 with Classpath Exception)
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3 only, as
 * published by the Free Software Foundation. cedarsoft GmbH designates this
 * particular file as subject to the "Classpath" exception as provided
 * by cedarsoft GmbH in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 3 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 3 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact cedarsoft GmbH, 72810 Gomaringen, Germany,
 * or visit www.cedarsoft.com if you need additional information or
 * have any questions.
 */

package com.cedarsoft.business.calc;

import com.cedarsoft.business.Money;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.joda.time.LocalDate;

import java.math.BigDecimal;

/**
 * Contains interest information within a given interval.
 */
public class InterestDetails {
  @NonNls
  public static final String PROPERTY_INTERVAL = "interval";
  @NonNls
  public static final String PROPERTY_BEGIN = "begin";
  @NonNls
  public static final String PROPERTY_END = "end";
  @NonNls
  public static final String PROPERTY_INTEREST = "interest";
  @NonNls
  public static final String PROPERTY_BASE_VALUE = "baseValue";
  @NonNls
  public static final String PROPERTY_DAYS = "days";
  @NonNls
  public static final String PROPERTY_RATE = "rate";

  @NotNull
  private final Money baseValue;
  @NotNull
  private final Money interest;
  private final double days;
  @NotNull
  private final LocalDate begin;
  @NotNull
  private final LocalDate end;
  private final double rate;

  /**
   * Creates a new instance
   *
   * @param begin
   * @param end
   * @param rate      the rate for this period
   * @param baseValue the baseValue for this period
   * @param interest  the interest for this period
   * @param days      the amount of days of this period (depending on the time system!)
   */
  public InterestDetails( @NotNull LocalDate begin, @NotNull LocalDate end, double rate, @NotNull Money baseValue, @NotNull Money interest, double days ) {
    this.begin = begin;
    this.end = end;
    this.rate = rate;
    this.baseValue = baseValue;
    this.interest = interest;
    this.days = days;
  }

  @NotNull
  public Money getBaseValue() {
    return baseValue;
  }

  public double getDays() {
    return days;
  }

  @NotNull
  public LocalDate getBegin() {
    return begin;
  }

  @NotNull
  public LocalDate getEnd() {
    return end;
  }

  @NotNull
  public Money getInterest() {
    return interest;
  }

  public double getRateAsDouble() {
    return rate;
  }

  @NotNull
  public BigDecimal getRate() {
    return new BigDecimal( rate );
  }

  @Override
  public String toString() {
    return "InterestDetails{" +
        "interest=" + interest +
        ", days=" + days +
        ", rate=" + rate +
        ", baseValue=" + baseValue +
        ", begin=" + begin +
        ", end=" + end +
        '}';
  }
}
